# OnlineCourse Product for Zope
# Copyright (C) 2001 Maurice Davis <mjd@networklogic.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# See file COPYING.txt
#
#
__doc__ = """Materials for the OnlineCourse Product."""

__version__ = '0.1.0'

from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager

from Globals import InitializeClass
from Globals import Persistent
from Globals import DTMLFile
from Globals import MessageDialog

from Acquisition import Implicit

from OFS.SimpleItem import Item
from OFS.Folder import Folder

from Products.MSWordDocument import MSWordDocument

import sys, os, string


################## Document #######################

class Document(
    MSWordDocument,
    RoleManager
    ):
    """Document object."""

#    __implements__ = IDocument

    meta_type='Document'

    security=ClassSecurityInfo()

    manage_options = (
	{'label': 'Properties','action':'editDocumentForm'},
	{'label': 'Upload',		'action': 'manage_uploadForm'},
	{'label': 'View',       'action': 'display'},
	)

    def __init__(self, id, file, title='', order='1'):
	MSWordDocument.__init__(self, id, file, title)
	self._order = order
	self._fname = file.filename

    security.declareProtected('Change OnlineCourse', 'editDocumentForm')
    addDocumentForm = DTMLFile('dtml/add/addDocument', globals())

    security.declareProtected('Change OnlineCourse', 'addDocument')
    def addDocument(self, file, title='', order='1', REQUEST=None):
	"""Web interface to add a Document instance to a Folder."""
	id = string.strip(title)
	id = string.lower(id)
	id = string.replace(id,' ', '_')
	self._setObject(id, Document(id, file, title, order))
	if REQUEST is not None:
	    if hasattr(REQUEST, 'idx'):
		return self.index_html(self, REQUEST)
	    else:
		return self.manage_main(self, REQUEST)
	
    security.declarePrivate('addDocument')
    _editDocumentForm = DTMLFile('dtml/edit/editDocument', globals())

    security.declareProtected('Change OnlineCourse', 'editDocumentForm')
    def editDocumentForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit Document Instance."""
	REQUEST['title'] = self.title
	REQUEST['order'] = self._order
	return self._editDocumentForm(self, REQUEST,
				      management_view='Properties',
				      manage_tabs_message=manage_tabs_message)


    security.declareProtected('Change OnlineCourse', 'editDocument')
    def editDocument(self, title, order, REQUEST=None):
	"""Edit Document Instance"""
	self.title = title
	self._order = order
	if REQUEST is not None:
	    return self.editDocumentForm(REQUEST,
					 manage_tabs_message=' Document Changed.')
	return None

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html =  DTMLFile('dtml/display/indexDocument', globals())

    security.declareProtected('View OnlineCourse', 'display')
    display =  DTMLFile('dtml/display/displayDocument', globals())

    security.declareProtected('Change OnlineCourse', 'manage_uploadForm')
    manage_uploadForm = DTMLFile('dtml/edit/uploadForm', globals())

    security.declareProtected('Change OnlineCourse', 'manage_upload')
    def manage_upload(self, file, REQUEST=None):
        "Upload the MSWord Document and process it (update_data)"
	data = file.read()
	self._fname = file.filename
	name = file.filename
        self.update_data(data, name)
                
        return MessageDialog(
            title = "Uploaded",
            message = "MSWord Document data for %s uploaded" % self.title,
            action = "manage_main",
            )

    security.declareProtected('View OnlineCourse', 'getDisplayOrderAsInt')
    def getDisplayOrderAsInt(self):
	"""Return the Display Order as an Integer"""
	return self._order

    security.declareProtected('View OnlineCourse', 'getDisplayOrderAsString')
    def getDisplayOrderAsString(self):
	"""Return the Display Order as a String"""
	return str(self._order)

    security.declareProtected('View OnlineCourse', 'getDocFileName')
    def getDocFileName(self):
	"""Return the Assigned Date"""
	if hasattr(self, '_fname'):
	    return self._fname

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(Document)


################## MaterialsFolder #######################

class MaterialsFolder(
    Implicit,
    RoleManager,
    Persistent,
    Folder
    ):
    """Container class for the course material documents."""

#    __implements__ = IMaterialsFolder

    meta_type='Materials Folder'

    security=ClassSecurityInfo()

    def all_meta_types(self):
	all = MaterialsFolder.inheritedAttribute('all_meta_types')(self)
	return ({'name':'Document', 'action': 'addDocumentForm'},
		)

    addDocumentForm = Document.addDocumentForm
    addDocument = Document.addDocument

    manage_options = (
	{'label': 'Contents', 'action': 'manage_main'},
	)

    security.declareProtected('Change OnlineCourse', 'addMaterialsFolder')
    def addMaterialsFolder(self, id, title='', REQUEST=None):
	"""Web interface to add an MaterialsFolder instance to a folder."""

	self._setObject(id, MaterialsFolder(id, title))
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    def __init__(self, id, title=''):
	self.id = id
	self.title = title

    security.declarePrivate('_editMaterialsFolderForm')
    _editMaterialsFolderForm = DTMLFile('dtml/edit/editMaterialsFolder', globals())

    security.declareProtected('Change OnlineCourse', 'editMaterialsFolderForm')
    def editMaterialsFolderForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit MaterialsFolder Instance."""
	return self._editMaterialsFolderForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)

    security.declareProtected('Change OnlineCourse', 'editMaterialsFolder')
    def editMaterialsFolder(self, title='', REQUEST=None):
	"""Edit MaterialsFolder Instance"""
	self.title = title
	if REQUEST is not None:
	    return self.editMaterialsFolderForm(REQUEST,
					     manage_tabs_message='Online Course Changed.')
	return None
    security.declareProtected('Change OnlineCourse', 'manage_workspace')
    security.declareProtected('Change OnlineCourse', 'manage_main')
    manage_workspace = manage_main = DTMLFile('dtml/display/mainMaterialsFolder', globals())

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html = DTMLFile('dtml/display/indexMaterialsFolder', globals())

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(MaterialsFolder)



