# OnlineCourse Product for Zope
# Copyright (C) 2001 Maurice Davis <mjd@networklogic.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# See file COPYING.txt
#
#
__doc__ = """Members Implementation for OnlineCourse Product."""

__version__ = '0.1.0'

from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager

from Globals import InitializeClass
from Globals import Persistent
from Globals import DTMLFile
from Globals import MessageDialog

from Acquisition import Implicit

from OFS.SimpleItem import Item
from OFS.Folder import Folder
from OFS.Folder import manage_addFolder
from HomeWork import HomeWorkFolder
from ParticipationLog import ParticipationLogFolder

from IOnlineCourse import IOnlineCourse, ICourseMember
import sys, os, string


################## Member #######################

class Member(
    Implicit,
    RoleManager,
    Persistent,
    Item
    ):
    """ Member object."""

#    __implements__ = IMember

    meta_type='Member'

    security=ClassSecurityInfo()

    manage_options = (
	{'label': 'Properties','action':'editMemberForm'},
	)


    def __init__(self, id, title='', last_name='', first_name='', mi='', password='',
		 phone_number='', email='', category='student'):
	self.id = id
	self.title = title
	self._last_name = last_name
	self._first_name = first_name
	self._mi = mi
	self._password = password
	self._phone_number = phone_number
	self._email = email
	self._category = category

    addHomeWorkFolder = HomeWorkFolder.addHomeWorkFolder
    addParticipationLogFolder = ParticipationLogFolder.addParticipationLogFolder
    addMemberForm = DTMLFile('dtml/add/addMember', globals())

    security.declareProtected('Change OnlineCourse', 'addMember')
    def addMember(self, last_name='', first_name='', mi='',
			password='', phone_number='', email='', category='Student',
			REQUEST=None):
	"""Web interface to add a Member instance to an OnlineCourse folder."""
	id = string.lower(first_name) + string.lower(last_name)
	title = last_name + ', ' + first_name + ' ' + mi
        #Set up acl_user
	if category == 'Instructor':
	    category = ('Instructor', 'Manager')
	if category == 'Student':
	    category = ('Student',)
	for (alid, obj) in self.objectItems('User Folder'):
	    if alid == 'acl_users':
		obj._doAddUser(id, password, category, ())
	self.member_info._setObject(id, Member(id, title, last_name, first_name, mi,
					       password, phone_number, email,
					       category))
	if category == ('Student',):
	    #Add student folder to homework folder
	    self.homework._setObject(id, HomeWorkFolder(id, title))
	    #Add student folder to participation log folder
	    self.participation_log._setObject(id, ParticipationLogFolder(id, title))
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    security.declarePrivate('_editMemberForm')
    _editMemberForm = DTMLFile('dtml/edit/editMember', globals())

    security.declareProtected('Change OnlineCourse', 'editMemberForm')
    def editMemberForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit Member Instance."""
	REQUEST['phone_number'] = self._phone_number
	REQUEST['email'] = self._email
	return self._editMemberForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)


    security.declareProtected('Change OnlineCourse', 'editMember')
    def editMember(self, phone_number='', email='', REQUEST=None):
	"""Edit Member Instance"""
	self._phone_number = phone_number
	self._email = email
	if REQUEST is not None:
	    return self.editMemberForm(REQUEST,
				       manage_tabs_message=' Member Changed.')
	return None

    security.declareProtected('View OnlineCourse', 'getFullName')
    def getFullName(self):
	"""Get a Member Full Name"""
	return self.title

    security.declareProtected('View OnlineCourse', 'getMemberID')
    def getMemberID(self):
	"""Get a Member' ID"""
	return self.id

    security.declareProtected('View OnlineCourse', 'getMemberPhone')
    def getMemberPhone(self):
	"""Get a Member' Phone Number"""
	return self._phone_number

    security.declareProtected('View OnlineCourse', 'getMemberEMail')
    def getMemberEMail(self):
	"""Get a Member EMail"""
	return self._email

    security.declareProtected('View OnlineCourse', 'getCategory')
    def getCategory(self):
	"""Student or Instructor"""
	return self._category

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(Member)

################## MembersFolder #######################

class MembersFolder(
    Implicit,
    RoleManager,
    Persistent,
    Folder
    ):
    """Container class for members."""

#    __implements__ = IMembersFolder

    meta_type='MembersFolder'

    security=ClassSecurityInfo()

    def all_meta_types(self):
	all = MembersFolder.inheritedAttribute('all_meta_types')(self)
	return ()

    manage_options = (
	{'label': 'Contents', 'action': 'manage_main'},
	)

    security.declareProtected('Change OnlineCourse', 'manage_workspace')
    security.declareProtected('Change OnlineCourse', 'manage_main')
    manage_workspace = manage_main = DTMLFile('dtml/display/mainMembersFolder', globals())

    security.declareProtected('Change OnlineCourse', 'addMembersFolder')
    def addMembersFolder(self, id, title='', REQUEST=None):
	"""Web interface to add an MembersFolder instance to a folder."""

	self._setObject(id, MembersFolder(id, title))
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    def __init__(self, id, title=''):
	self.id = id
	self.title = title

    security.declarePrivate('_editMembersFolderForm')
    _editMembersFolderForm = DTMLFile('dtml/edit/editMembersFolder', globals())

    security.declareProtected('Change OnlineCourse', 'editMembersFolderForm')
    def editMembersFolderForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit MembersFolder Instance."""
	return self._editMembersFolderForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)

    security.declareProtected('Change OnlineCourse', 'editMembersFolder')
    def editMembersFolder(self, title='', REQUEST=None):
	"""Edit MembersFolder Instance"""
	self.title = title
	if REQUEST is not None:
	    return self.editMembersFolderForm(REQUEST,
					      manage_tabs_message='Member Folder Changed.')
	return None

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html =  DTMLFile('dtml/display/indexMembersFolder', globals())

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(MembersFolder)
