# OnlineCourse Product for Zope
# Copyright (C) 2001 Maurice Davis <mjd@networklogic.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# See file COPYING.txt
#
#
__doc__ = """OnlineCourse Product."""

__version__ = '0.1.0'

from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from AccessControl.Role import RoleManager
from AccessControl import User

from Globals import InitializeClass
from Globals import Persistent
from Globals import DTMLFile
from Globals import HTMLFile
from Globals import MessageDialog

from Acquisition import Implicit

from OFS.SimpleItem import Item
from OFS.Folder import Folder, manage_addFolder
from OFS.DTMLDocument import addDTMLDocument

from Members import Member
from Members import MembersFolder
from Materials import MaterialsFolder
from Assignments import AssignmentsFolder
from Messages import MessagesFolder
from ChatSessions import ChatSessionsFolder
from utils import Login
from utils import SimpleFolder

from IOnlineCourse import IOnlineCourse, ICourseMember

from DateTime import DateTime
from ImageFile import ImageFile

import sys, os, string


################## OnlineCourse #######################

class OnlineCourse(
    Implicit,
    RoleManager,
    Persistent,
    Folder
    ):
    """Container class for OnlineCourse objects."""

    __implements__ = IOnlineCourse

    meta_type='Online Course'

    security=ClassSecurityInfo()

    def all_meta_types(self):
	all = OnlineCourse.inheritedAttribute('all_meta_types')(self)
	return ({'name':'Member','action': 'addMemberForm'},
		)

    security.declareProtected('Change OnlineCourse', 'addOnlineCourseForm')
    addOnlineCourseForm = DTMLFile('dtml/add/addOnlineCourse', globals())

    def addOnlineCourse(self, id, title='', last_name='', first_name='', mi='',
			password='', phone_number='', email='', REQUEST=None):
	"""Web interface to add an OnlineCourse instance to a folder."""

	self._setObject(id, OnlineCourse(id, title, last_name, first_name, mi,
					 password, phone_number, email))
	for (i, o) in self.objectItems('Online Course'):
	    if i == id:
		o.base_url = o.absolute_url()
		break
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    addMemberForm = Member.addMemberForm
    addMember = Member.addMember

    addMembersFolder = MembersFolder.addMembersFolder
    addMaterialsFolder = MaterialsFolder.addMaterialsFolder
    addAssignmentsFolder = AssignmentsFolder.addAssignmentsFolder
    addMessagesFolder = MessagesFolder.addMessagesFolder
    addChatSessionsFolder = ChatSessionsFolder.addChatSessionsFolder
    addLogin = Login.addLogin
    addSimpleFolder = SimpleFolder.addSimpleFolder

    manage_options = (
	{'label': 'Contents', 'action': 'manage_main'},
	{'label': 'Properties','action':'editOnlineCourseForm'},
	) +  RoleManager.manage_options

    css_content = """body {  font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 8pt; color: #000000}
a:link {  font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 8pt; color: #006600}
a:hover {  font-family: Verdana, Arial, Helvetica, sans-serif; color: #00CC00; font-size: 8pt}
a:visited {  font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 8pt; color: #CC9900}
td {  font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 8pt; color: #000000}
th {  font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 9pt; color: #000000}
h1 {  font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 10pt; color: #006600}"""


    def __init__(self, id, title='', last_name='', first_name='', mi='',
		 password='', phone_number='', email=''):
	self.id = id
	self.title = title
	self.addSimpleFolder('CSS', title='Style Sheets Folder',
			     meta_types={'name':'Style Sheet','action': 'addDTMLDocumentForm'})
	self.CSS.addDTMLDocument(id='temp.css', title='temp.css', file=self.css_content)
	self.addDTMLDocument(id='getting_started_text', title='', file='Getting started text.')
	self.addDTMLDocument(id='welcome_text', title='', file='Welcome text.')
	self.manage_addUserFolder()
	(ob_id, ob) = self.objectItems('User Folder')[0]
	ob._addRole('Instructor')
	ob._addRole('Student')
	self.addMembersFolder('member_info', 'Instructor and Student Info')
	self.addMaterialsFolder('course_materials', 'Course Materials')
	self.addAssignmentsFolder('assignments', 'Assignments')
	self.addSimpleFolder('homework', 'Home Work')
	self.addMessagesFolder('messages', 'Messages')
	self.addChatSessionsFolder('chat_sessions', 'Chat Sessions')
	self.addSimpleFolder('participation_log', 'Participation Log')
	self.addLogin(id='cookie_authentication')
	category = 'Instructor'
	self.addMember(last_name, first_name, mi, password, phone_number,
		       email, category)

    security.declarePrivate('_editOnlineCourseForm')
    _editOnlineCourseForm = DTMLFile('dtml/edit/editOnlineCourse', globals())

    security.declareProtected('Change OnlineCourse', 'editOnlineCourseForm')
    def editOnlineCourseForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit OnlineCourse Instance."""
	return self._editOnlineCourseForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)

    security.declareProtected('Change OnlineCourse', 'editOnlineCourse')
    def editOnlineCourse(self, title='', REQUEST=None):
	"""Edit OnlineCourse Instance"""
	self.title = title
	if REQUEST is not None:
	    return self.editOnlineCourseForm(REQUEST,
					     manage_tabs_message='Online Course Changed.')
	return None

    security.declareProtected('View OnlineCourse', 'course_header_html')
    course_header_html =  DTMLFile('dtml/display/course_header', globals())

    security.declareProtected('View OnlineCourse', 'course_footer_html')
    course_footer_html =  DTMLFile('dtml/display/course_footer', globals())

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html =  HTMLFile('dtml/display/indexOnlineCourse', globals())

    security.declareProtected('View OnlineCourse', 'manage_top_frame')
    manage_top_frame =  DTMLFile('dtml/display/manage_top_frame', globals())

    security.declarePublic('login_form')
    login_form = HTMLFile('dtml/display/login_form', globals())

    security.declareProtected('View OnlineCourse', 'logged_in')
    logged_in = DTMLFile('dtml/display/logged_in', globals())

    security.declareProtected('View OnlineCourse', 'logout')
    def logout(self, REQUEST):
	"""Log the Member out."""
	self.recordLogOut()
	resp = REQUEST['RESPONSE']
	resp.expireCookie('__ac', path='/')
	iself = getattr(self, 'aq_inner', self)
	parent = getattr(iself, 'aq_parent', None)
	page = getattr(self, 'logged_out', None)
	if page is not None:
	    resp.redirect('logged_out')

    security.declarePublic('logged_out')
    logged_out = DTMLFile('dtml/display/logged_out', globals())

    security.declareProtected('View OnlineCourse', 'getting_started')
    getting_started = DTMLFile('dtml/display/getting_started', globals())

    security.declareProtected('View OnlineCourse', 'recordLogIn')
    def recordLogIn(self):
	"""Write User log in to Participation log"""
	member_id = self.getMemberID()
	if member_id != 'Not a Member'and 'Student' in self._getMemberRoles():
	    f = self.participation_log._getOb(member_id)
	    f.addParticipationLog(member_id, 'Log In')

    security.declareProtected('View OnlineCourse', 'recordLogOut')
    def recordLogOut(self):
	"""Write User log out to Participation log"""
	member_id = self.getMemberID()
	if member_id != 'Not a Member'and 'Student' in self._getMemberRoles():
	    f = self.participation_log._getOb(member_id)
	    f.addParticipationLog(member_id, 'Log Out')

    security.declareProtected('View OnlineCourse', 'getMemberID')
    def getMemberID(self):
	"""Get the Member ID."""
	u = getSecurityManager().getUser()
	for (mid, mob) in self.member_info.objectItems('Member'):
	    if mob.id == u.getUserName():
		return mob.id
	return 'Not a Member'

    security.declarePublic('getMemberName')
    def getMemberName(self):
	"""Return a members full name"""
	member_id = self.getMemberID()
	for (mid, mob) in self.member_info.objectItems('Member'):
	    if member_id == mob.getMemberID():
		return mob.getFullName()

    security.declareProtected('View OnlineCourse', 'getBaseURL')
    def getBaseURL(self):
	"""Return a members full name"""
	return self.base_url

    security.declarePrivate('_getMemberRoles')
    def _getMemberRoles(self):
	"""Get the member roles as tuple."""
	u = getSecurityManager().getUser()
	return u.getRoles()

    security.declareProtected('View OnlineCourse', 'memberIsInstructor')
    def memberIsInstructor(self):
	if 'Instructor' in self._getMemberRoles():
	    return 1
	else:
	    return 0

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

    cellback_img = ImageFile('Products/OnlineCourse/images/cellback.gif')
    back_img = ImageFile('Products/OnlineCourse/images/back.gif')
    assignments_img = ImageFile('Products/OnlineCourse/images/assignments.gif')
    assignmentsa_img = ImageFile('Products/OnlineCourse/images/assignmentsa.gif')
    homework_img = ImageFile('Products/OnlineCourse/images/homework.gif')
    homeworka_img = ImageFile('Products/OnlineCourse/images/homeworka.gif')
    members_img = ImageFile('Products/OnlineCourse/images/members.gif')
    membersa_img = ImageFile('Products/OnlineCourse/images/membersa.gif')
    participation_img = ImageFile('Products/OnlineCourse/images/participation.gif')
    participationa_img = ImageFile('Products/OnlineCourse/images/participationa.gif')
    chatroom_img = ImageFile('Products/OnlineCourse/images/chatroom.gif')
    chatrooma_img = ImageFile('Products/OnlineCourse/images/chatrooma.gif')
    message_img = ImageFile('Products/OnlineCourse/images/message.gif')
    messagea_img = ImageFile('Products/OnlineCourse/images/messagea.gif')
    course_img = ImageFile('Products/OnlineCourse/images/course.gif')
    coursea_img = ImageFile('Products/OnlineCourse/images/coursea.gif')
    conatact_img = ImageFile('Products/OnlineCourse/images/contact.gif')
    conatacta_img = ImageFile('Products/OnlineCourse/images/contacta.gif')
    getting_img = ImageFile('Products/OnlineCourse/images/getting.gif')
    gettinga_img = ImageFile('Products/OnlineCourse/images/gettinga.gif')
    login_img = ImageFile('Products/OnlineCourse/images/login.gif')
    logina_img = ImageFile('Products/OnlineCourse/images/logina.gif')
    logout_img = ImageFile('Products/OnlineCourse/images/logout.gif')
    logouta_img = ImageFile('Products/OnlineCourse/images/logouta.gif')
    home_img = ImageFile('Products/OnlineCourse/images/home.gif')
    homea_img = ImageFile('Products/OnlineCourse/images/homea.gif')
    management_img = ImageFile('Products/OnlineCourse/images/management.gif')
    managementa_img = ImageFile('Products/OnlineCourse/images/managementa.gif')
    banner_img = ImageFile('Products/OnlineCourse/images/banner.gif')
    top_img = ImageFile('Products/OnlineCourse/images/top.gif')
    bottom_img = ImageFile('Products/OnlineCourse/images/bottom.gif')

InitializeClass(OnlineCourse)

