# OnlineCourse Product for Zope
# Copyright (C) 2001 Maurice Davis <mjd@networklogic.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# See file COPYING.txt
#
__doc__ = """ParticipationLog Implementation."""

__version__ = '0.1.0'

from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager

from Globals import InitializeClass
from Globals import Persistent
from Globals import DTMLFile
from Globals import MessageDialog

from Acquisition import Implicit

from OFS.SimpleItem import Item
from OFS.Folder import Folder

from DateTime import DateTime

#from IOnlineCourse import IOnlineCourse, IMessage
import sys, os, string, whrandom


################## ParticipationLog #######################



class ParticipationLog(
    Implicit,
    RoleManager,
    Persistent,
    Item
    ):
    """ParticipationLogs object."""

#    __implements__ = IParticipationLog

    meta_type = 'Participation Log'

    security = ClassSecurityInfo()

    manage_options = (
	)


    def __init__(self, id, action=''):
	self.id = id
	self._action = action

    security.declareProtected('Change OnlineCourse', 'addParticipationLog')
    def addParticipationLog(self, member_id, action, REQUEST=None):
	"""Web interface to add a ParticipationLog instance to an OnlineCourse folder."""
	id = DateTime().strftime('%m-%d-%Y-%I-%M-%S%p')
	folder_ob = self.participation_log._getOb(member_id)
	folder_ob._setObject(id, ParticipationLog(id, action))
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    security.declareProtected('Change OnlineCourse', 'getAction')
    def getAction(self):
	return self._action

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(ParticipationLog)


################## ParticipationLogFolder #######################

class ParticipationLogFolder(
    Implicit,
    RoleManager,
    Persistent,
    Folder
    ):
    """Container class for participation logs."""

#    __implements__ = IParticipationLogFolder

    meta_type='Participation Log Folder'

    security=ClassSecurityInfo()

    def all_meta_types(self):
	return ()

    security.declareProtected('Change OnlineCourse', 'addParticipationLog')
    addParticipationLog = ParticipationLog.addParticipationLog

    manage_options = (
	{'label': 'Contents', 'action': 'manage_main'},
	)

    security.declareProtected('Change OnlineCourse', 'manage_workspace')
    security.declareProtected('Change OnlineCourse', 'manage_main')
    manage_workspace = manage_main = DTMLFile('dtml/display/mainParticipationLogFolder', globals())

    security.declareProtected('Change OnlineCourse', 'addParticipationLogFolder')
    def addParticipationLogFolder(self, id, title='', REQUEST=None):
	"""Web interface to add an ParticipationLogFolder instance to a folder."""

	self._setObject(id, ParticipationLogFolder(id, title))
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    def __init__(self, id, title='', username=''):
	self.id = id
	self.title = title
	self._username = username

    security.declarePrivate('_editParticipationLogFolderForm')
    _editParticipationLogFolderForm = DTMLFile('dtml/edit/editParticipationLogFolder', globals())

    security.declareProtected('Change OnlineCourse', 'editParticipationLogFolderForm')
    def editParticipationLogFolderForm(self, REQUEST=None, management_view='', manage_tabs_message=''):
	"""Form to edit ParticipationLogFolder Instance."""
	return self._editParticipationLogFolderForm(self, REQUEST,
					  management_view='Properties',
					  manage_tabs_message=manage_tabs_message)

    security.declareProtected('Change OnlineCourse', 'editParticipationLogFolder')
    def editParticipationLogFolder(self, title='', REQUEST=None):
	"""Edit ParticipationLogFolder Instance"""
	self.title = title
	if REQUEST is not None:
	    return self.editParticipationLogFolderForm(REQUEST,
					     manage_tabs_message='Online Course Changed.')
	return None

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html =  DTMLFile('dtml/display/indexParticipationLogFolder', globals())

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))

InitializeClass(ParticipationLogFolder)

