# OnlineCourse Product for Zope
# Copyright (C) 2001 Maurice Davis <mjd@networklogic.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# See file COPYING.txt
#
#
from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager

from Globals import InitializeClass
from Globals import Persistent

from Acquisition import Implicit

from OFS.Folder import Folder

from base64 import encodestring
from urllib import quote, unquote
from DateTime import DateTime
from ZPublisher import BeforeTraverse
import Globals
from Globals import HTMLFile
from Globals import DTMLFile
#from zLOG import LOG, ERROR

# Constants.
ATTEMPT_NONE = 0
ATTEMPT_LOGIN = 1
ATTEMPT_CONT = 2
NEED_LOGIN = 3

from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem

class SimpleItemWithProperties (PropertyManager, SimpleItem):
    '''
    A common base class for objects with configurable
    properties in a fixed schema.
    '''
    manage_options = (
        PropertyManager.manage_options
        + SimpleItem.manage_options)


    security = ClassSecurityInfo()
    security.declarePrivate(
        'manage_addProperty',
        'manage_delProperties',
        'manage_changePropertyTypes',
        )

    def manage_propertiesForm(self, REQUEST, *args, **kw):
        'An override that makes the schema fixed.'
        my_kw = kw.copy()
        my_kw['property_extensible_schema__'] = 0
        return apply(PropertyManager.manage_propertiesForm,
                     (self, self, REQUEST,) + args, my_kw)

    security.declarePublic('propertyLabel')
    def propertyLabel(self, id):
        """Return a label for the given property id
        """
        for p in self._properties:
            if p['id'] == id:
                return p.get('label', id)
        return id




class Login (SimpleItemWithProperties):
    '''
    Reads cookies during traversal and simulates the HTTP
    authentication headers.
    '''
    meta_type = 'Login'

    security = ClassSecurityInfo()

    security.declarePrivate('delRequestVar')
    def delRequestVar(self, req, name):
        try: del req.other[name]
        except: pass
        try: del req.form[name]
        except: pass
        try: del req.cookies[name]
        except: pass
        try: del req.environ[name]
        except: pass

    security.declarePrivate('setAuthCookie')
    def setAuthCookie(self, resp, ac):
        resp.setCookie('__ac', quote(ac), path='/')

    security.declarePrivate('modifyRequest')
    def modifyRequest(self, req, resp):
        if not req._auth:
            if (req.has_key('__ac_password') and
                req.has_key('__ac_name')):
                name = req['__ac_name']
                pw = req['__ac_password']
                ac = encodestring('%s:%s' % (name, pw))
                req._auth = 'basic %s' % ac
                resp._auth = 1
                e = (DateTime() + 365).toZone('GMT').rfc822()
		resp.setCookie('__ac_name', name, path='/', expires=e)
                self.setAuthCookie(resp, ac)
		self.delRequestVar(req, '__ac_name')
                self.delRequestVar(req, '__ac_password')
                return ATTEMPT_LOGIN
            elif req.has_key('__ac'):
                ac = unquote(req['__ac'])
                req._auth = 'basic %s' % ac
                resp._auth = 1
                self.delRequestVar(req, '__ac')
                return ATTEMPT_CONT
	    else:
		return NEED_LOGIN
	return ATTEMPT_NONE

    def __call__(self, container, req):
        '''The __before_publishing_traverse__ hook.'''
	resp = req['RESPONSE']
        attempt = self.modifyRequest(req, resp)
	if (attempt == ATTEMPT_LOGIN or attempt == NEED_LOGIN):
	    page = getattr(container, 'login_form', None)
	    if page is not None:
		# Provide a login page.
		req._hold(ResponseCleanup(resp))
		resp.unauthorized = self.unauthorized
		resp._unauthorized = self._unauthorized


    security.declarePublic('credentialsChanged')
    def credentialsChanged(self, user, name, pw):
        resp = self.REQUEST['RESPONSE']
        ac = encodestring('%s:%s' % (name, pw))
        self.setAuthCookie(resp, ac)

    def _cleanupResponse(self):
        resp = self.REQUEST['RESPONSE']
        try: del resp.unauthorized
        except: pass
        try: del resp._unauthorized
        except: pass
        return resp

    security.declarePublic('unauthorized')
    def unauthorized(self):
	"""doc"""
        url = self.getLoginURL()
        if url:
            raise 'Redirect', url
        # Use the standard unauthorized() call.
        resp = self._cleanupResponse()
        resp.unauthorized()

    security.declarePublic('_unauthorized')
    def _unauthorized(self):
	"""Doc"""
        url = self.getLoginURL()
        if url:
            resp = self.REQUEST['RESPONSE']
            resp.redirect(url, lock=1)
            # We don't need to raise an exception.
            return
        # Use the standard _unauthorized() call.
        resp = self._cleanupResponse()
        resp._unauthorized()

    security.declarePublic('getLoginURL')
    def getLoginURL(self):
        '''
        Redirects to the login page.
        '''
        if 'login_form':
            req = self.REQUEST
            resp = req['RESPONSE']
            iself = getattr(self, 'aq_inner', self)
            parent = getattr(iself, 'aq_parent', None)
            page = getattr(parent, 'login_form', None)
            if page:
                retry = getattr(resp, '_auth', 0) and '1' or ''
                url = '%s/login_form' % (req['URL1'])
                return url
        return None

    security.declareProtected('View OnlineCourse', 'logout')
    def logout(self):
        '''
        Logs out the user and redirects to the logout page.
        '''
	resp = self.REQUEST['RESPONSE']
	resp.expireCookie('__ac', path='/')
	iself = getattr(self, 'aq_inner', self)
	parent = getattr(iself, 'aq_parent', None)
	page = getattr(self, 'logged_out', None)
	if page is not None:
	    resp.redirect('logged_out')


    # Installation and removal of traversal hooks.

    def manage_beforeDelete(self, item, container):
        if item is self:
            handle = self.meta_type + '/' + self.id
            BeforeTraverse.unregisterBeforeTraverse(container, handle)

    def manage_afterAdd(self, item, container):
        if item is self:
            handle = self.meta_type + '/' + self.id
            container = container.this()
            nc = BeforeTraverse.NameCaller(self.id)
            BeforeTraverse.registerBeforeTraverse(container, nc, handle)

    def addLogin(self, id, REQUEST=None):
	' '
	ob = Login()
	ob.id = id
	self._setObject(id, ob)
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))


InitializeClass(Login)


class ResponseCleanup:
    def __init__(self, resp):
        self.resp = resp

    def __del__(self):
        # Free the references.
        try: del self.resp.unauthorized
        except: pass
        try: del self.resp._unauthorized
        except: pass
        try: del self.resp
        except: pass

InitializeClass(ResponseCleanup)


################## SimpleFolder #######################

class SimpleFolder(
    Implicit,
    RoleManager,
    Persistent,
    Folder
    ):
    """Simple Container class."""

#    __implements__ = ISimpleFolder

    meta_type='Simple Folder'

    security=ClassSecurityInfo()

    def all_meta_types(self):
	all = SimpleFolder.inheritedAttribute('all_meta_types')(self)
	return (self._folder_meta_types,)

    manage_options = (
	{'label': 'Contents', 'action': 'manage_main'},
	)

    def addSimpleFolder(self, id, title='', meta_types={}, REQUEST=None):
	"""Web interface to add an SimpleFolder instance to a folder."""

	self._setObject(id, SimpleFolder(id, title, meta_types))
	if REQUEST is not None:
	    return self.manage_main(self, REQUEST)

    def __init__(self, id, title='', meta_types={}):
	self.id = id
	self.title = title
	self._folder_meta_types = meta_types

    security.declareProtected('Change OnlineCourse', 'manage_main')
    security.declareProtected('Change OnlineCourse', 'manage_workspace')
    manage_main = manage_workspace = DTMLFile('dtml/display/mainSimpleFolder', globals())

    security.declareProtected('View OnlineCourse', 'index_html')
    index_html = DTMLFile('dtml/display/indexSimpleFolder', globals())

    security.setPermissionDefault('View OnlineCourse', ('Manager', 'Instructor', 'Student'))
    security.setPermissionDefault('Change OnlineCourse', ('Manager', 'Instructor'))


InitializeClass(SimpleFolder)
